#!/usr/bin/perl
#use diagnostics;
use warnings;
use strict;
use Getopt::Long;
use Net::SMTP::SSL;
use Net::SMTPS;
use DBI;
use Config::IniFiles;
use Net::IP qw(:PROC);
use utf8;
use Encode qw(decode encode);
use DateTime::Format::Mail;
use Email::MessageID;
my ($opt_h,$opt_V, $opt_D, $opt_hello);
my $opt_verbose = 1;
my $config="/etc/esia/esia.conf";
my $errorid;
my $debug = 0;
my $to;
my $opt_ssl;
sub Trace
{
    if($debug)
    {
        print encode("utf-8","DEBUG: ".$_[0]."\n");
    }
}
sub print_help
{
    my $string = "SEND MAIL: envoie un email d'alerte\n";
    $string .="\t-e | --error         <string>  identifiant de l'erreur dans la base de données\n";
    $string .="\t                               possibilité de spécifier \"test\" pour tester les paramètres\n";
    $string .="\t                               possibilité de spécifier \"min\" pour tester avec une erreur base de données\n";
    $string .="\t-t | --to            <string>  liste des emails destinataires (séparé par des ','\n";
    $string .="\t-c | --config        <path>    fichier de configuration d'Esia (défaut: /etc/esia/esia.conf\n";
    $string .="\t-d | --disable-hello           Désactive l'envoi du Hello\n";
    $string .="\t-s | --ssl                     force l'activation SSL\n";
    $string .="\t-D | --debug                   active le mode débogage\n";
    $string .="\t-v | --verbose       <integer> modifie le niveau de verbosité du mode debug (défaut: 1) si égal à 2, il active le debug du client SMTP\n";
    $string .="\t-h | --help                    affiche cette aide\n";
    print encode("utf-8",$string);
}
sub format_mail {
	my $txt = $_[0];
	my $dat = $_[1];
	#my $ip = new Net::IP ($dat->{device_device_ip_address }) or die (Net::IP::Error());
	my $sip = $dat->{device_ip_address};
    my $errormsg = $dat->{happen_message};    
    $errormsg =~ s/\|.*//gi;
	my $service_lc = lc($dat->{service_name});
    	$txt =~ s/\%node.group[^d]/$dat->{device_group_name}/gi;
	$txt =~ s/\%node.groupdesc/$dat->{device_group_description}/gi;
	$txt =~ s/\%node.name/$dat->{device_name}/gi;
	$txt =~ s/\%node.ip/$sip/gi;
	$txt =~ s/\%node.description/$dat->{device_description}/gi;
	$txt =~ s/\%service.name/$service_lc/gi;
	$txt =~ s/\%service.parameter/$dat->{service_parameter}/gi;
	$txt =~ s/\%error.type/$dat->{error_type}/gi;
	$txt =~ s/\%error.name/$dat->{error_name}/gi;
	$txt =~ s/\%error.time/$dat->{happen_timestamp}/gi;
	$txt =~ s/\%error.message/$errormsg/gi;
	$txt =~ s/\\n/\n/gi;
	if($dat->{error_type} eq 'critical')
	{
		$txt =~ s/\%color/#DA002E/gi;
	}
	elsif($dat->{error_type} eq 'warning')
	{
		$txt =~ s/\%color/#FFCB1F/gi;
	}
	elsif($dat->{error_type} eq 'unknwon')
	{
		$txt =~ s/\%color/#575759/gi;
	}
        elsif($dat->{error_type} eq 'ok')
        {
                $txt =~ s/\%color/#83BC00/gi;
        }
	else
	{
		$txt =~ s/\%color/#FFFFFF/gi;
	}
	return $txt;
#Noeud : %node.name : %node.ip \n Service: %service   ETAT: %error.type Heure: %error.time \n %error.message
}

#test des paramètres
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"D"   => \$opt_D,	"debug"       => \ $opt_D,
	"d"   => \$opt_hello,"--disable-hello"=> \$opt_hello,
    "s"   => \$opt_ssl,"--ssl"=> \$opt_ssl,
	"e=s" => \$errorid,	"error=s" => \$errorid,
	"t=s" => \$to,	"to=s" => \$to,
	"c=s" => \$config,"config=s" => \$config,
	"v=i" => \$opt_verbose,	"verbose=i" => \$opt_verbose,
);
# -h & --help print help
if ($opt_h)
{ 
    print "Usage : send_mail.pl -e <id_error> -t <to> -c <esia config file>\n"; 
    print_help(); 
    exit(1); 
}
if(!$errorid)
{
    print "no error is defined\n"; 
    print_help(); 
    exit(1); 
}
if($opt_D)
{ 
  $debug = 1;
}
if(!$to)
{
  print "mail to not defined\n"; 
  exit(1);
}
Trace("Chargement du fichier de configuration: $config");
#chargement du fichier de configuration
my $cfg = new Config::IniFiles( -file => $config  )or die "esia config file not found (default: /etc/esia/esia.conf)";
#Connexion à la base de donnée
Trace("Connexion à la base de donnée Postgresql");
my $db = DBI -> connect($cfg->val( 'POSTGRESQL', 'PGSQL_connect' ),$cfg->val( 'POSTGRESQL', 'PGSQL_username' ),$cfg->val( 'POSTGRESQL', 'PGSQL_pwd' )) or die "Connection DB Failed";
#connexion au serveur SMTP

my $host;
my $port=25;
my $smtp;
my $user;
my $password;
my $from;
my $message;
my $subject;
my $query;

if( $errorid eq 'min')
{
    Trace("Mode Testing activé ($errorid)");
	$query=$db->prepare("SELECT MIN(happen_id) as hi_min from esia_error_happen;");
	$query->execute();
	$errorid = $query->fetchrow_hashref->{hi_min};
}
Trace("ID de l'erreur selectionné $errorid");
$query = $db->prepare("SELECT config_key,config_value
                          FROM esia_configuration
                          WHERE config_key LIKE 'MAIL_%';");
$query->execute();
while (my @data = $query->fetchrow_array ) {
	if($data[0] eq 'MAIL_SMTP')
	{
		$host = $data[1];
	}
	elsif($data[0] eq 'MAIL_SMTP_PORT')
	{
		$port = $data[1];
	}
	elsif($data[0] eq 'MAIL_USER')
	{
		$user = $data[1];
	}
	elsif($data[0] eq 'MAIL_PASSWORD')
	{
		$password = $data[1];
	}
	elsif($data[0] eq 'MAIL_FROM')
	{
		$from = $data[1];
	}
	elsif($data[0] eq 'MAIL_FORMAT_MESSAGE')
	{
		$message = $data[1];
	}
	elsif($data[0] eq 'MAIL_FORMAT_SUBJECT')
	{
		$subject = $data[1];
	}
}

my $hello = $from =~ s/@/\./r;
if($opt_hello)
{ 
    $hello= undef;
}
my $debug_smtp=0;
if($opt_verbose == 2)
{
    $debug_smtp=1;
}
Trace("Connexion au serveur SMTP: $host:$port user:$user from:$from helo:$hello");
if($port == '465' || $opt_ssl)
{
    Trace("Secure mode");
	#connexion sécurisée
	$smtp = Net::SMTP::SSL->new(Host =>$host,Hello=>$hello ,Port=> $port,Timeout => 120,Debug=>$debug_smtp) or die "Could not connect to server";
}
elsif( $port == '587' && $host =~ m/office365/ )
{
    Trace("office365 mode");
	#connexion office365
	$smtp = Net::SMTPS->new(Host =>$host,Port=> $port,Timeout => 120,doSSL => 'starttls',Debug=>$debug_smtp) or die "Could not connect to server";
}
else
{
    Trace("unsecure mode");
	#connexion classique
	$smtp = Net::SMTP->new(Host =>$host,Hello=>$hello ,Port=> $port,Timeout => 120,Debug=>$debug_smtp) or die "Could not connect to server";
}
if(defined $user)
{
    Trace("Authentification");
    $smtp->auth($user ,$password) or die "Could not authenticate $! \n quit";
}
Trace("Connexion SMTP success");
if($errorid eq 'test')
{
    Trace("Envoit email de test");

    Trace("Format et encodage de l'email");
    $message = encode('UTF-8', "Votre test d'envoi d'email a reussi");
    $subject = encode('UTF-8', "Test alerting esia");
    Trace("to $to subject $subject");
    Trace("Envoi de l'email");
	$smtp->mail("$from");
	$smtp->to( split (/[,;]/,$to));
	$smtp->data();
	$smtp->datasend("To: $to\n");
	$smtp->datasend("From: $from\n");
    	my $date_send = DateTime::Format::Mail->format_datetime( DateTime->now() );
    	Trace("Date: $date_send");
    	$smtp->datasend("Date: $date_send\n");
	$smtp->datasend("Content-Type: text/html; charset=\"UTF-8\"\n");
	$smtp->datasend("User-Agent: Mozilla/5.0 (X11; Linux x86_64; rv:68.0) Gecko/20100101\nThunderbird/68.10.0\n");
	$smtp->datasend("Subject: $subject\n");
	$smtp->datasend("\n");
	$smtp->datasend("$message\n");
	$smtp->dataend();
	$smtp->quit();
    print encode("utf-8","envoi reussi\n");
}
else
{
    #récupération des informations sur l'erreur
    $query = $db->prepare("SELECT device_name,device_ip_address,device_description,device_description,error_name,error_type,to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS tz') AS happen_timestamp,happen_message,
				    service_name,service_parameter,esia_device_group.device_group_name as device_group_name,device_group_description as device_group_description,happen_acquitte,4 as device_ip_version
                              FROM esia_error_happen
                              JOIN esia_device ON happen_device_id = device_id
                              JOIN esia_errors ON happen_error_id= error_id
                              JOIN esia_services ON happen_service_id= service_id
                              LEFT JOIN esia_device_group_in ON group_in_device_id=device_id
                              LEFT JOIN esia_device_group ON device_group_id=group_in_device_group_id
                              WHERE happen_id=?");
    $query->execute($errorid);
    my $data = $query->fetchrow_hashref;
    if($data)
    {
        Trace("Recupération de l'erreur reussie");
        if($data->{happen_acquitte} == 1)
        {
            $data->{error_type}="ok";
            $data->{happen_message}=": status changed to ok";
        }
        Trace("Format et encodage de l'email");
	    $subject = format_mail($subject,$data);
    #	$subject = encode("MIME-Header",$subject);
	    $message = format_mail($message,$data);
            $message = encode('UTF-8', $message);
#            $subject = encode('UTF-8', $subject);
	    $subject = encode('MIME-Header',$subject);
            Trace("to $to subject $subject");
            Trace("Envoi de l'email");
	    $smtp->mail("$from");
	    $smtp->to( split (/[,;]/,$to));
	    $smtp->data();
	    $smtp->datasend("To: $to\n");
	    $smtp->datasend("From: $from\n");
            $smtp->datasend("Subject: $subject\n");	
	    my $new_mid = Email::MessageID->new( host => 'esia-sa.com' );
	    my $mid = $new_mid->in_brackets;
            $smtp->datasend("Message-ID:$mid\n");
	    my $date_send = DateTime::Format::Mail->format_datetime( DateTime->now() );
	    Trace("Date: $date_send");
            $smtp->datasend("Date: $date_send\n");
	    $smtp->datasend("Content-Type: text/html; charset=\"UTF-8\"\n");
	    $smtp->datasend("User-Agent: Esia V1.0");
	    
	    $smtp->datasend("\n");
	    $smtp->datasend("$message\n<CR><LF>.<CR><LF>");
	    $smtp->dataend();
	    $smtp->quit();
        Trace("Envoi reussi");
    }
    else
    {
	    print "NO DATA FOUND => CHECK error_happen_id please\n";
    }
    $query->finish;
    $db->disconnect;
}
exit(0);
